@@ -1,12 +1,13 @@
 import numpy as np
 import pyfftw
 from multiprocessing import cpu_count
+import time
 
 def osconvolve(signal, kernel, *, mode='full', nfft=None,
                threads=cpu_count(), axis=-1, outarray=None,
                input_index_bounds=None, output_index_bounds=None,
                describe_dims=False, ds=None, input_dim_restrictions=None,
-               output_offset=0):
+               output_offset=0, verbose=False):
     """Computes a convolution with FFTW, using the convolution
     theorem. This function has been written to minimize memory
     usage.
@@ -77,14 +78,16 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
     else:
         first_ind = (tot_length - outsize) // 2
         last_ind = first_ind + outsize
-    print(f"First ind {first_ind}, last ind {last_ind}, diff {last_ind - first_ind}, total size {tot_length}")
+#     print(f"First ind {first_ind}, last ind {last_ind}, diff {last_ind - first_ind}, total size {tot_length}")
     
     downsample = False
     if ds is not None:
         if divmod(ds, 1)[1] != 0:
             raise ValueError("'ds' factor must be an integer > 1")
+        ds = int(ds)
         block_offset = 0
         downsample = True
+    
     ###############################################################
     # handle output params
     if real_output:
@@ -118,23 +121,21 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
     
     expected_shape = tuple(expected_shape)
     if describe_dims:
-        print(f"Output array should have shape {expected_shape} and dtype {dtype}")
+        if verbose:
+            print(f"Output array should have shape {expected_shape} and dtype {dtype}")
         return expected_shape, dtype
 
     if outarray is None:
         outarray = np.zeros(expected_shape, dtype=dtype)
-        print(f"Allocated array of shape {outarray.shape} with dtype {dtype}")
+        if verbose:
+            print(f"Allocated array of shape {outarray.shape} with dtype {dtype}")
     else:
-#         if output_offset == 0:
-#             if outarray.shape != expected_shape:
-#                 raise ValueError(f"Output array is expected to have shape"
-#                                  f" {expected_shape} but got {outarray.shape}")
-#         else:
-        print("Checking output array shape is disabled, make sure portion of"
-              f" output array has shape {expected_shape}")
-        if outarray.dtype != dtype:
-            raise TypeError(f"Output array should have dtype {dtype}"
-                            f" but got {outarray.dtype}")
+        if verbose:
+            print(
+                "Checking output array shape is disabled, make sure portion of"
+                f" output array has shape {expected_shape}")
+        if not real_output and np.isrealobj(outarray[..., 0:1]):
+            raise TypeError("Output array is real but expected one of a complex dtype")
 
     ######################################################################
     if nfft is not None:
@@ -178,7 +179,7 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
                               flags=['FFTW_ESTIMATE'], 
                               threads=threads)
 
-    y_slices = [np.s_[:]] * signal.ndim
+    y_slices = [0] * signal.ndim
     y_slices[axis] = np.s_[0:M]
     y[tuple(y_slices)] = kernel
     y_slices[axis] = np.s_[M:nfft]
@@ -216,10 +217,6 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
     bounds = np.arange(start_offset, start_offset + (div+2)*L, L)
     block_bounds = np.vstack((bounds[:-1], bounds[1:])).T
     n_blocks = block_bounds.shape[0]
-
-    # remove when ready to deploy
-    #a = np.zeros((n_blocks, nfft), dtype='<c16')
-    #xs = np.zeros((n_blocks, nfft), dtype='<c16')
     
     signal_slices_1 = [np.s_[:]] * signal.ndim
     signal_slices_2 = [np.s_[:]] * signal.ndim
@@ -244,20 +241,26 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
     # are written. The outarray_marker keeps track of that.
     first_block_to_check, first_offset = divmod(first_ind, L)
     last_block_to_check, last_offset = divmod(last_ind, L)
-    print(f'first index {first_ind}, last index {last_ind}')
-    print(f"First segment: {M-1}, second segment {L}, for a total of {nfft} FFT points")
-    print(f"Total number of blocks: {n_blocks}")
-    print(f'Index {first_ind}, first block to check: {first_block_to_check}')
-    print(f"Index {last_ind} last block to check: {last_block_to_check}")
-    print(f'First offset {first_offset}, last offset {last_offset}')
+    # print(f'first index {first_ind}, last index {last_ind}')
+    # print(f"First segment: {M-1}, second segment {L}, for a total of {nfft} FFT points")
+    # print(f"Total number of blocks: {n_blocks}")
+    # print(f'Index {first_ind}, first block to check: {first_block_to_check}')
+    # print(f"Index {last_ind} last block to check: {last_block_to_check}")
+    # print(f'First offset {first_offset}, last offset {last_offset}')
 
     tot_samples = 0
+    tr = 0 # read time
+    tp = 0 # processing time
+    tw = 0 # write time
+
     for ii, (start, stop) in enumerate(block_bounds):
+        t0 = time.time()
 
         if ii < first_block_to_check or ii > last_block_to_check:
             pass
         else:
-#             print(f"Computing block {ii} of {n_blocks - 1}")
+            tr0 = time.time()
+            # print(f"Computing block {ii} of {n_blocks - 1}")
             # initialize entire block to 0, then fill with
             # appropriate input data if it exists
             x[:] = 0 
@@ -287,21 +290,25 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
                     x[tuple(x_slices_1)] = signal_chunk
                 except:
                     # M - 1 segment already initialized to 0, so leave as is
-                    print(f"Requested {ind1} out of bounds, doing nothing")
+#                     print(f"Requested {ind1} out of bounds, doing nothing")
+                    pass
 
             # fill L segment of block
             signal_slices_2[axis] = np.s_[start:stop]
             signal_chunk = signal[tuple(signal_slices_2)]
             length = signal_chunk.shape[axis]
             x_slices_2[axis] = np.s_[M-1:M-1 + length]
+
             x[tuple(x_slices_2)] = signal_chunk
+            tr += time.time() - tr0
 
-
+            tp0 = time.time()
             fft_sig(normalise_idft=True)
             conv[:] = x * y
             fft_conv_inv(normalise_idft=True)
 
             if ii == first_block_to_check and ii == last_block_to_check:
+
                 if downsample:
                     conv_slices[axis] = np.s_[M-1+first_offset:M-1+last_offset:ds]
                     n_samples = conv[tuple(conv_slices)].shape[axis]
@@ -310,7 +317,9 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
                     n_samples = last_offset - first_offset
                     conv_slices[axis] = np.s_[M-1+first_offset:M-1+last_offset]
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker+n_samples]
+
             elif ii == first_block_to_check:
+
                 if downsample:
                     conv_slices[axis] = np.s_[M-1+first_offset::ds]
                     
@@ -321,29 +330,33 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
                         block_offset = ds - rem
                     else:
                         block_offset = 0
-                    print(f"Wrote {n_samples} samples. Leftover {rem}. Next offset {block_offset}")
+#                     print(f"Wrote {n_samples} samples. Leftover {rem}. Next offset {block_offset}")
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker+n_samples]
                 else:
                     n_samples = nfft - (M - 1 + first_offset)
-                    #print(f'Block {ii}, taking {n_samples} samples')
+#                     print(f'Block {ii}, taking {n_samples} samples')
                     conv_slices[axis] = np.s_[M - 1 + first_offset:nfft]
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker+n_samples]
+
             elif ii == last_block_to_check:
+
                 if downsample:
-                    print("last block to check!")
-                    print(f"Searching range {M-1+block_offset} to {M-1+last_offset}")
+#                     print("last block to check!")
+#                     print(f"Searching range {M-1+block_offset} to {M-1+last_offset}")
                     conv_slices[axis] = np.s_[M-1+block_offset:M-1+last_offset:ds]
                     n_samples = conv[tuple(conv_slices)].shape[axis]
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker+n_samples]
                 else:
-                    #print(f"Block {ii}, last block to check")
+#                     print(f"Block {ii}, last block to check")
                     n_samples = last_offset
-                    #print(f'Block {ii}, taking {n_samples} samples')
-                    #n_samples = outarray.shape[axis] - outarray_marker
-                    print(f"Searching range {M-1} to {M-1+n_samples}")
+#                     print(f'Block {ii}, taking {n_samples} samples')
+#                     n_samples = outarray.shape[axis] - outarray_marker
+#                     print(f"Searching range {M-1} to {M-1+n_samples}")
                     conv_slices[axis] = np.s_[M-1:M-1 + n_samples]
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker + n_samples]
+
             else:
+
                 if downsample:
                     conv_slices[axis] = np.s_[M-1+block_offset:nfft:ds]
                     
@@ -355,23 +368,37 @@ def osconvolve(signal, kernel, *, mode='full', nfft=None,
                     else:
                         block_offset = 0
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker + n_samples]
-                    print(f"Wrote {n_samples} samples. Leftover {rem}. Next offset {block_offset}")
-                    
-
+#                     print(f"Wrote {n_samples} samples. Leftover {rem}. Next offset {block_offset}")
                 else:
-                    
-                    print(f'Block {ii} of {n_blocks-1}, taking {n_samples} samples')
+#                     print(f'Block {ii} of {n_blocks-1}, taking {n_samples} samples')
                     n_samples = stop - start
                     outarray_slices[axis] = np.s_[outarray_marker:outarray_marker + n_samples]
                     conv_slices[axis] = np.s_[M-1:nfft]
 
+            tp += time.time() - tp0
+
+            tw0 = time.time()
             if real_output:
                 outarray[tuple(outarray_slices)] = conv[tuple(conv_slices)].real
             else:
                 outarray[tuple(outarray_slices)] = conv[tuple(conv_slices)]
             outarray_marker += n_samples
             tot_samples += n_samples
+            tw += time.time() - tw0
 
-    print(f"Wrote {tot_samples} data points")
-    assert expected_shape[axis] == tot_samples
+        if verbose:
+            print(
+                f"Computed block {ii} of {n_blocks - 1}, "
+                f"elapsed time: {time.time() - t0} seconds")
+
+#     print(f"Wrote {tot_samples} data points")
+    if not expected_shape[axis] == tot_samples:
+        raise ValueError(f"Expected to write {expected_shape[axis]} samples "
+                         f"for axis {axis} but actually wrote {tot_samples}")
+
+    if verbose:
+        print(f"Total read time: {tr} seconds")
+        print(f"Total processing time: {tp} seconds")
+        print(f"Total write time: {tw} seconds")
+    
     return outarray
\ No newline at end of file
